#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <gwine@free.fr>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <gwine@free.fr>
##############################################################################
##############################################################################
## Define Gwine::Config class
#############################################################################
package Gwine::Config;
use strict;
use warnings;

BEGIN {
    use Gnome2::GConf;
    use vars qw( @ISA %fields $AUTOLOAD $permitted_fields );
    $permitted_fields = '_permitted_fields';
}

#################################################################
## Default values are used by old config style (rc style)
## There are useless since use of GConf because default values
## are contained in gconf schemas.
%fields = (
	   # Informations to save on disk
	   # Last opened file
	   opened_file => undef,
	   # say if opened_file must be used
	   save_opened_file => 0,
	   # Size of main windown (geometry = ${width}x$height)
	   geometry => undef,
	   # Position of divider between panes
	   paned_pos => undef,
	   # say if geometry and hpaned_pos must be used
	   save_geometry => 0, 
	   # Use confirmation popups (or not)
	   conf_popups => 1,
	   # Hide wines with no bottles
	   hide_wines => 0,
	   # Columns that must be visible in wine list
	   wine_visible => [1, 1, 1, 1, 1, 1, 1, 1, 0],
	   # Columns that must be visible in prod list
	   prod_visible => [1, 1, 1, 1, 1, 0],
	   # Wine List sorted column 
	   wlist_sort_column => 0,
	   # Wine List sorting order
	   wlist_sort_order => 'ascending',
	   # Prod List sorted column 
	   plist_sort_column => 0,
	   # Prod List sorting order
	   plist_sort_order => 'ascending',
	   # View style (Wines only, prods only or tree view)
	   view_style => 2,
	   # HOME directory
	   home => undef,
	   # save last clicked between lists or tree
	   last_clicked => undef,
	   # save icon path
	   icon_path => undef,
	   # save gladexml path
	   xml_path => undef,
	   );

# List of fields to be saved on disk by save method
# These fields may be set by restore method
my @to_save = qw( opened_file save_opened_file paned_pos geometry 
                  save_geometry conf_popups hide_wines wine_visible 
                  prod_visible wlist_sort_column wlist_sort_order 
                  plist_sort_column plist_sort_order view_style );

# format for each key to save. When key is a list, format of elements's type is 
# stored as Gnome2::GConf retrieves key's perl format
my @format = qw( string bool int string bool bool bool bool bool int 
                 string int string int );


my $notify_id;


##############################################################################
## Create a new Gwine::Config object
##############################################################################
sub new {
  my $this = shift;
  my $class = ref($this) || $this;

  my $self = {
      $permitted_fields => \%fields, %fields
  };
  bless $self, $class;

  my $client = Gnome2::GConf::Client->get_default;
  $client->add_dir('/apps/gwine', 'preload-none');
  $notify_id = $client->notify_add('/apps/gwine', "$class\::on_gconf_notify_event");

  return $self;
}


sub DESTROY {
  my $client = Gnome2::GConf::Client->get_default;
  $client->notify_remove($notify_id);
  $client->remove_dir('/apps/gwine');
    
}

##############################################################################
## AUTOLOAD
##############################################################################
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self)
        or die "$self is not an object so we cannot '$AUTOLOAD'\n",
            "We were called from ".join(", ", caller)."\n\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;       # strip fully-qualified portion

    if (exists $self->{$permitted_fields}{$name} ) {
        # This allows dynamic data methods - see hash fields above
        # eg $class->UI('new_value');
        # or $current_value = $class->UI;
        if (@_) {
            return $self->{$name} = shift;
        } else {
            return $self->{$name};
        }

    } else {
        die "Can't access method `$name' in class $type\n".
            "We were called from ".join(", ", caller)."\n\n";
    }
}


##############################################################################
## visibility methods
##############################################################################
sub wine_visible {
    my $self = shift;

    if (@_) { 
	$self->{wine_visible} = shift;
	if (defined $self->{wv_handler}) {
	    &{$self->{wv_handler}}(@{$self->{wine_visible}})
	}
	return $self->{wine_visible}
    }
    else { return $self->{wine_visible} }
    
}


sub prod_visible {
    my $self = shift;

    if (@_) { 
	$self->{prod_visible} = shift;
	if (defined $self->{pv_handler}) {
	    &{$self->{pv_handler}}(@{$self->{prod_visible}})
	}
	return $self->{prod_visible}
    }
    else { return $self->{prod_visible} }
    
}


##############################################################################
## get_opened_file. Take care of save_opened_file
##############################################################################
sub get_opened_file {
    my $self = shift;
    
    if ($self->{save_opened_file}) { return $self->{opened_file} }
    else { return }
}

##############################################################################
## get_geometry. Take care of save_geometry
##############################################################################
sub get_geometry {
    my $self = shift;
    
    if ($self->{save_geometry}) { return $self->{geometry} }
    else { return }
}

##############################################################################
## Restore Config
##############################################################################
sub restore {
    my $self = shift;

    my $conf = Gnome2::GConf::Client->get_default;
    my $key = '/apps/gwine';
    my @save_format = @format;

    if ($conf = Gnome2::GConf::Client->get_default) {
	foreach my $item (@to_save) {
	    my $format = shift @save_format;
	    my $href = $conf->get("$key/$item");
	    $self->{$item} = $href->{value} if defined $href->{value};
	}
    }
    else { return 0 }
    return 1;
}


##############################################################################
## Save Config
##############################################################################
sub save  {
    my $self = shift;
    
    my $conf;
    my $key = '/apps/gwine';
    my @save_format = @format;

    if ($conf = Gnome2::GConf::Client->get_default) {
	foreach my $item (@to_save) {
	    my $format = shift @save_format;
	    $conf->set("$key/$item",
		       { type => $format, value => $self->{$item} });
	}
    }
    else { return 0 }

    # Delete $HOME/.gwinerc if exists
    my $file = $self->{home} . '/.gwinerc';
    unlink $file if (-e $file);

    return 1
}


######################################################"
## GConf events
######################################################"
sub on_gconf_notify_event {
    my ($client, $cnx_id, $entry) = @_;
    
    return unless $entry->{value};
    my $key = (split m!/!, $entry->{key})[-1];
    $fields{$key} = $entry->{value}->{data}
}


######################################################"
## Set handlers for visibility
######################################################"
sub set_handler_for_wine_visible {
    my $self = shift;
    $self->{wv_handler} = shift;
}


sub set_handler_for_prod_visible {
    my $self = shift;
    $self->{pv_handler} = shift;
}


1;

__END__
