#!/usr/bin/perl
##############################################################################
# Copyright (c) 2005
# Author Sylvain Daubert <gwine@free.fr>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <gwine@free.fr>
#############################################################################
require 5.8.1;
use strict;
use warnings;

package DialogPref;

BEGIN {
    use Encode;
    use Gtk2::GladeXML;
    use Locale::TextDomain qw( gwine );
    use Gwine::Config;
}

use constant FALSE => 0;
use constant TRUE  => 1;


##################################################################
## helper function
##################################################################
sub __u { decode('UTF-8', __($_[0])) }
sub __xu { decode('UTF-8', __x(@_)) }


#############################################################
## Callbacks
#############################################################

sub on_toggled_event {
  my ($cell, $path_str, $model) = @_;
  my $path = Gtk2::TreePath->new ($path_str);
    
  # get toggled iter
  my $iter = $model->get_iter ($path);
  my ($value) = $model->get ($iter, 1);
  # do something with the value
  $value ^= 1;
  # set new value
  $model->set ($iter, 1, $value);
}


sub on_response_event {
    my $dialog = shift;
    my $rep = shift;
    my $p = shift;
    my $xml = $p->[0];
    my $config = $p->[1];

    if (($rep eq 'cancel') or ($rep eq 'delete-event')) {
	$dialog->destroy;
	return
    }
    
    # rep is 'ok' or 'apply'
    # Get new settings
    $config->save_opened_file($xml->get_widget('cbutton_lfile')->get_active);
    $config->save_geometry($xml->get_widget('cbutton_size')->get_active);
    $config->conf_popups($xml->get_widget('cbutton_popup')->get_active);

    # If hide_wines has changed, wine list is refreshed
    my $old_hide_wines = $config->hide_wines;
    $config->hide_wines($xml->get_widget('cbutton_zerowine')->get_active);
    AppGwine::list_wines() if ($old_hide_wines != $config->hide_wines);

    # Get wine visibility info
    my $wstore = $xml->get_widget('tview1')->get_model;
    my $iter = $wstore->get_iter_first;
    my $i = 0;
    my @tab = ();
    do {
	$tab[$i] = $wstore->get_value($iter, 1);
	$i++;
    } while ($iter = $wstore->iter_next($iter));
    $config->wine_visible(\@tab);

    # Get prod visibility info
    my $pstore = $xml->get_widget('tview2')->get_model;
    $iter = $pstore->get_iter_first;
    $i = 0;
    my @tab2 = ();
    do {
	$tab2[$i] = $pstore->get_value($iter, 1);
	$i++;
    } while ($iter = $pstore->iter_next($iter));
    $config->prod_visible(\@tab2);

    $config->save;

    $dialog->destroy if $rep eq 'ok'
}


#############################################################
## Methods
#############################################################

sub new {    
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = {};
    bless $self, $class;
}


sub create {
    # Create new dialog
    my $self = new(shift);
    my $cfg = shift;
    my $class = ref $self;

    my $path = $cfg->xml_path;
    my $xml = Gtk2::GladeXML->new("$path/gwine.glade", 'dialog_pref', 
				  'gwine');
    $self->{xml} = $xml;
    $xml->signal_autoconnect_from_package('Gwine::DialogPref');
    $xml->get_widget('dialog_pref')->signal_connect('response', 
						    "$class\::on_response_event", 
						    [$xml, $cfg]);

    # Set Treeview1 Model (wines visibility)
    my $tview = $xml->get_widget('tview1');
    my $store = Gtk2::ListStore->new('Glib::String', 'Glib::Boolean');
    $tview->set_model($store);
    my $renderer = Gtk2::CellRendererText->new;
    my $col = Gtk2::TreeViewColumn->new_with_attributes(__u('Columns'), 
							$renderer,
							'text' => 0);
    $tview->append_column($col);
    $renderer = Gtk2::CellRendererToggle->new;
    $renderer->signal_connect (toggled => "$class\::on_toggled_event", 
			       $store);
    $col = Gtk2::TreeViewColumn->new_with_attributes(__u('Visible'), 
							$renderer,
							'active' => 1);
    $tview->append_column($col);
    $tview->get_selection->set_mode('single');

    # Set Treeview2 Model (producers visibility)
    $tview = $xml->get_widget('tview2');
    $store = Gtk2::ListStore->new('Glib::String', 'Glib::Boolean');
    $tview->set_model($store);
    $renderer = Gtk2::CellRendererText->new;
    $col = Gtk2::TreeViewColumn->new_with_attributes(__u('Columns'), 
						     $renderer,
						     'text' => 0);
    $tview->append_column($col);
    $renderer = Gtk2::CellRendererToggle->new;
    $renderer->signal_connect (toggled => "$class\::on_toggled_event", 
			       $store);
    $col = Gtk2::TreeViewColumn->new_with_attributes(__u('Visible'), 
						     $renderer,
						     'active' => 1);
    $tview->append_column($col);
    $tview->get_selection->set_mode('single');

    # Init all check boxes
    $xml->get_widget('cbutton_lfile')->set_active($cfg->save_opened_file);
    $xml->get_widget('cbutton_size')->set_active($cfg->save_geometry);
    $xml->get_widget('cbutton_popup')->set_active($cfg->conf_popups);
    $xml->get_widget('cbutton_zerowine')->set_active($cfg->hide_wines);
    my $wstore = $xml->get_widget('tview1')->get_model;
    my @wine_cols = (AppGwine->wine_cols)[0 .. 7];
    foreach (my $i = 0; $i < @wine_cols; $i++) {
	my $ins_iter = $wstore->append;
	$wstore->set($ins_iter,
		     0 => $wine_cols[$i],
		     1 => $cfg->wine_visible->[$i])
    }
    my $pstore = $xml->get_widget('tview2')->get_model;
    my @prod_cols = AppGwine->prod_cols;
    pop @prod_cols;
    for (my $i = 0; $i < @prod_cols; $i++) {
	my $ins_iter = $pstore->append;
	$pstore->set($ins_iter,
		     0 => $prod_cols[$i],
		     1 => $cfg->prod_visible->[$i])
    }

    return $self
}


sub xml { return $_[0]->{xml} }
sub top { return $_[0]->{xml}->get_widget('dialog_pref') }


1;

__END__
