#!/usr/bin/perl
##############################################################################
# Copyright (c) 2005
# Author Sylvain Daubert <gwine@free.fr>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <gwine@free.fr>
#############################################################################
## This file contains handlers for gwine UI
#############################################################################
require 5.8.1;
use strict;
use warnings;

package DialogWine;

BEGIN {
    use Encode;
    use Gtk2::GladeXML;
    use Locale::TextDomain qw( gwine );
}

use constant FALSE => 0;
use constant TRUE  => 1;


##################################################################
## helper function
##################################################################
sub __u { decode('UTF-8', __($_[0])) }
sub __xu { decode('UTF-8', __x(@_)) }

#############################################################
## Internal functions
#############################################################

my $copy_treestore = sub {
    my $model = shift;
    my $path = shift;
    my $iter = shift;
    my $data = shift; # contains Dialog_wine's treestore
    
    # Producers's node is not copied
    return FALSE if $path->to_string eq '1';

    my $par_path = $path->copy;
    $par_path->up;
    $par_path = $par_path->to_string;
    if (defined $par_path) {
	$par_path = '' if $par_path eq '0';
	$par_path =~ s/0:(\d*)/$1/;
    }
    # Wines's node is not copied
    else { return FALSE }

    my @values = $model->get($iter);
    my $parent = ($par_path ne '') ? $data->get_iter_from_string($par_path) : undef;
    my $new_iter = $data->append($parent);
    my $i = 0;
    $data->set($new_iter, map {$i++ => $_} @values);
    # Function returns FALSE not to stop foreach
    return FALSE
};


sub add_item {
    my ($dxml, $date, $type, $nb, $comment) = @_;

    my $store = $dxml->get_widget('ltreeview')->get_model;
    my $ins_iter = $store->append;
    $store->set($ins_iter,
		0 => $date,
		1 => $type,
		2 => $nb,
		3 => $comment);
}


# Arg : resp (ok or cancel)
# Return a ref on a Gwine::GWine
sub create_wine_from_dialog {
    my $xml = shift;
    my $db = shift;

    my $lh = Gwine::L10N->get_l10n();

    my $domain = $xml->get_widget('entry_domain')->get_text;
    my $vintage = $xml->get_widget('entry_vintage')->get_text;
    my $color = $xml->get_widget('cb_color')->get_active;
    my $bdate = $xml->get_widget('entry_bdate')->get_text;
    my $nb = $xml->get_widget('entry_quantity')->get_text;
    my $locale_size = $xml->get_widget('entry_size')->get_text;
    my $size = $lh->numunf($locale_size);
    my $locale_price = $xml->get_widget('entry_price')->get_text;
    my $price = $lh->numunf($locale_price);
    my $prod_name = $xml->get_widget('cb_prod')->get_model->get($xml->get_widget('cb_prod')->get_active_iter);
    my $prod = $db->prod->by_name($prod_name);
    $prod = $db->prod->get_id($prod);
    my $cons = $xml->get_widget('entry_cons')->get_text;
    
    my $text_buf = $xml->get_widget('txtview_notes')->get_buffer;
    my $start_iter = $text_buf->get_start_iter;
    my $end_iter = $text_buf->get_end_iter;
    my $notes = $text_buf->get_text($start_iter, $end_iter, FALSE);
    
    my $selection = $xml->get_widget('ztreeview2')->get_selection;
    my $store = $xml->get_widget('ztreeview2')->get_model;
    my (undef, $iter) = $selection->get_selected;
    my ($type, $id);
    if (defined $iter) {
	($type, $id) = split m!/!, $store->get_value($iter, 2);
    }
    else { $type = 'undef' }
    
    # Some checking...
    ### Check date format
    my $format = __u('\d{1,2}-\d{1,2}-\d{4}');
    if ($bdate !~ m|$format|) {
	$xml->get_widget('entry_bdate')->set_text(__u('mm-dd-yyyy'));
	$xml->get_widget('entry_bdate')->select_region(0, -1);
	return
    }
    ### Check Appellation
    if (($type ne 'appel') and ($type ne 'ext')) {
	my $popup = Gtk2::MessageDialog->new($xml->get_widget('app_gwine'),
					     'modal', 'info', 'close',
					     __u('No appellation selected on tree'));
	$popup->run;
	$popup->destroy;
	return
    }
    # Get cellar book
    my $bstore = $xml->get_widget('ltreeview')->get_model;
    my $biter = $bstore->get_iter_first;
    my @cellar;
    while ($biter) {
	my @line = $bstore->get($biter);
	push @cellar, \@line;
	$biter = $bstore->iter_next($biter);
    }
    # Create new wine
    my $wine = Gwine::GWine->new;
    $wine->set(domain => $domain, 
	       vintage => $vintage, 
	       color => $color, 
	       date => $bdate, 
	       number => $nb, 
	       size => $size, 
	       price =>$price, 
	       appellation => $id, 
	       producer => $prod,
	       conservation => $cons,
	       notes => $notes);
    foreach (@cellar) {
	my $number = $_->[2];
	$number = -$number if $_->[1] eq __u('Out');
	$wine->book_add(date => $_->[0],
			number => $number,
			comment => $_->[3])
    }
    return $wine;
}


#############################################################
## Callbacks
#############################################################

sub on_button_add_press_event {
    my ($button, $event, $dxml) = @_;

    return if $event->button != 1;

    my $date = $dxml->get_widget('entry_date')->get_text;
    my $op = $dxml->get_widget('cb_op')->get_active;
    my $bottles = $dxml->get_widget('entry_bottle')->get_text;
    my $comment = $dxml->get_widget('entry_comment')->get_text;

    # Tests
    my $msg;
    my $regex = __u('\d{1,2}-\d{1,2}-\d{4}');
    $msg = __u('You must enter a number of bottles') 
	if $bottles !~ /\d+/;
    $msg = __u('Date is not valid') 
	if $date !~ m|$regex|;

    if (defined $msg) {
	my $msgbox = Gtk2::MessageDialog->new($dxml->get_widget('dialog_wine'),
					      'destroy-with-parent',
					      'info', 'close',
					      $msg);
	$msgbox->run;
	$msgbox->destroy;
	return
    }
    my $ope = (__u('In'), __u('Out'))[$op];
    add_item($dxml, $date, $ope, $bottles, $comment);
    return 0
}

sub on_button_remove_press_event {
    my ($button, $event, $dialog) = @_;
    
    return if $event->button != 1;
    $dialog->del_last_item;
    return 0
}


sub on_response_event {
    my $dialog = shift;
    my $rep = shift;
    my $p = shift;
    my ($xml, $db, $wine_id) = @$p;

    if (($rep eq 'cancel') or ($rep eq 'delete-event')) {
	$dialog->destroy;
	return
    }
    
    # rep is 'ok' or 'apply'
    my $wine = create_wine_from_dialog($xml, $db);
    if (defined $wine) {
	if (defined $wine_id) {
	    # Modifying an existing wine
	    # Add it to database
	    $wine->data($db->get_zone_path($wine_id));
	    $db->wine->by_id($wine_id, $wine);
	    # Refresh list
	    AppGwine::list_wines();
	}
	else { # Creating a new wine
	    # Add it to database
	    $wine_id = $db->wine->add($wine);
	    $wine->data($db->get_zone_path($wine_id));
	    # Add wineucer to wine list
	    AppGwine::add_wine($wine, $db);
	    if ($rep eq 'apply') {
		# If 'apply', then reconnect callback with wine_id 
		# because dialog becomes a modify dialog
		$dialog->signal_handlers_disconnect_by_func('DialogWine::on_response_event');
		$dialog->signal_connect('response', 
					"DialogWine::on_response_event", 
					[$xml, $db, $wine_id]);
	    }
	}
    } else { return }

    $dialog->destroy if $rep eq 'ok'
}


#############################################################
## Methods
#############################################################

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = {};
    bless $self, $class;
}


sub create {
    # Create new dialog
    my $self = new(shift);
    my $path = shift;
    my $db = shift;
    my $wine_id = shift;
    my $class = ref $self;

    my $xml = Gtk2::GladeXML->new("$path/gwine.glade", 'dialog_wine',
				  'gwine');
    $self->{xml} = $xml;
    $xml->signal_autoconnect_from_package('DialogWine');
    $xml->get_widget('add_button')->signal_connect('button-press-event', 
						   "$class\::on_button_add_press_event", 
						   $xml);
    $xml->get_widget('remove_button')->signal_connect('button-press-event', 
						      "$class\::on_button_remove_press_event", 
						      $xml);
    $xml->get_widget('dialog_wine')->signal_connect('response', 
						    "$class\::on_response_event", 
						    [$xml, $db, $wine_id]);

    my %h = @_;

    # Set Title
    $xml->get_widget('dialog_wine')->set_title($h{title});
    delete $h{title};

    # Init ztreeview2
    my $store = $h{xml}->get_widget('ztreeview')->get_model;
    my $local_store = Gtk2::TreeStore->new('Gtk2::Gdk::Pixbuf', 'Glib::String', 
					   'Glib::String');
    # Copy Global store (from app_gwine) to local one
    $store->foreach($copy_treestore, $local_store);
    $xml->get_widget('ztreeview2')->set_model($local_store);
    # Creation of Columns
    my $renderer1 = Gtk2::CellRendererPixbuf->new;
    my $col1 = Gtk2::TreeViewColumn->new_with_attributes('', $renderer1, 'pixbuf', 0);
    $xml->get_widget('ztreeview2')->append_column($col1);
    my $renderer2 = Gtk2::CellRendererText->new;
    my $col2 = Gtk2::TreeViewColumn->new_with_attributes('', $renderer2, 'text', 1);
    $xml->get_widget('ztreeview2')->append_column($col2);
    my $select = $xml->get_widget('ztreeview2')->get_selection;
    $select->set_mode('single');

    delete $h{xml};

    # Init ltreeview
    my @cols = (__u('Date'), __u('Operation'), __u('Bottles'), __u('Observations'));
    my @col_types = ('Glib::String', 'Glib::String', 'Glib::Int', 'Glib::String');
    my $lstore = Gtk2::ListStore->new(@col_types);
    $xml->get_widget('ltreeview')->set_model($lstore);
    my $i = 0;
    foreach (@cols) {
	my $renderer = Gtk2::CellRendererText->new;
	my $col = Gtk2::TreeViewColumn->new_with_attributes($_, 
							    $renderer,
							    'text',$i++);
	$xml->get_widget('ltreeview')->append_column($col);
    }

    # Set others settings
    foreach (keys %h) {
	my $w = $xml->get_widget($_);
	if (/cb_/) {
	    my @a = @{$h{$_}};
	    my $active = shift @a;
	    $w->append_text($_) foreach @a;
	    $w->set_active($active);
	    next
	}
	if (/entry_/) {
	    $w->set_text($h{$_})
	}
	if (/txtview_/) {
	    my $text_buf = $xml->get_widget($_)->get_buffer;
	    my $end_iter = $text_buf->get_end_iter;
	    $text_buf->insert($end_iter, $h{$_});
	}
	if (/cellar_book/) {
	    my @cellar = @{$h{$_}};
	    my $store = $xml->get_widget('ltreeview')->get_model;
	    foreach my $item (@cellar) {
		my $ins_iter = $store->append;
		$store->set($ins_iter,
			    0 => $item->[0],
			    1 => $item->[1],
			    2 => $item->[2],
			    3 => $item->[3]);
	    }
	}
    }

    return $self
}


sub xml { return $_[0]->{xml} }
sub top { return $_[0]->{xml}->get_widget('dialog_wine') }



1;

__END__
