#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <gwine@free.fr>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <gwine@free.fr>
#############################################################################
## Define GWine class
#############################################################################
## This class defines a wine data base record
#############################################################################
package Gwine::GWine;
use strict;
use warnings;

BEGIN {
    use Hash::Util qw/ lock_keys /;
    use vars qw( @ISA %fields $AUTOLOAD $permitted_fields );
    $permitted_fields = '_permitted_fields';
}

#############################################################################
## Fields :
##    domain       : string    : wine domain
##    color        : string    : wine color
##    date         : string    : buying date
##    number       : int       : number of bottles
##    size         : float     : size of bottles
##    price        : float     : price of one bottle
##    appellation  : string    : appellation Id
##    producer     : int       : producer Id
##    conservation : int       : years before drinking
##    notes        : string    : notes about this wine
##    book         : array ref : cellar book (list of entries about bottles movement)
#############################################################################
%fields = (
	   domain       => "",
	   vintage      => "",
	   color        => 0,
	   date         => "",
	   number       => 0,
	   calc_nb      => 0,
	   size         => '0.75',
	   price        => '0.0',
	   appellation  => undef,
	   producer    => -1,
	   conservation => "",
	   notes        => "",
	   data         => undef,
	   book         => undef,
	  );

#################
## Books

# Keys for a book entry. No more key may be used
my @book_entry_keys = ('date', 'number', 'comment');


#############################################################################
## Creator
#############################################################################
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = {
		$permitted_fields => \%fields,
		%fields,
	       };
    $self->{book} = [];
    bless($self,$class);
    return $self;
}


##############################################################################
## AUTOLOAD
##############################################################################
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self)
        or die "$self is not an object so we cannot '$AUTOLOAD'\n",
            "We were called from ".join(", ", caller)."\n\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;       # strip fully-qualified portion

    if (exists $self->{$permitted_fields}{$name} ) {
        if (@_) {
            return $self->{$name} = shift;
        } else {
            return $self->{$name};
        }

    } else {
        die "Can't access method `$name' in class $type\n".
            "We were called from ".join(", ", caller)."\n\n";
    }
}


##############################################################################
## DESTROY
##############################################################################
sub DESTROY {}


##############################################################################
## Set number of bottles (first entry, ie bought bottles)
## This number is stored in number whereas calc_nb is computed from
## number and from cellar book entries
##############################################################################
sub number {
    my $self = shift;
    if (@_) {
	$self->{calc_nb} = shift;
	$self->{number} = $self->{calc_nb};
	$self->{calc_nb} += $_->{number} foreach @{$self->{book}};
    }
    return $self->{number};
}

##############################################################################
## Return real number of bottles (calc_nb)
##############################################################################
sub real_number {
    my $self = shift;
    return $self->{calc_nb}
}


##############################################################################
## Same as vintage...
##############################################################################
sub millesime {
    my $self = shift;
    if (@_) {$self->{vintage} = shift}
    return $self->{vintage};
}


##############################################################################
## set multiple fields at once
##############################################################################
sub set {
    my $self = shift;
    my %hash = @_;

    foreach my $key (keys %hash) {
	if (exists $self->{$permitted_fields}{$key} ) {
	    if ($key eq 'number') { $self->number($hash{$key}) }
	    else { $self->{$key} = $hash{$key} }
	}
    }
}


##############################################################################
## Print all fields
##############################################################################
sub print {
    my $self = shift;
    my @keys = keys %{$self->{$permitted_fields}};
    
    foreach my $key (@keys) {
	print $key, ': ', $self->{$permitted_fields}{$key}, "\n"
    }
}


##############################################################################
##############################################################################
## Access to cellar book
##############################################################################

##############################################################################
## Add an entry
## An entry is composed by multiple fields:
##    - date   : date of operation (in machine time)
##    - number : number of bottles in operation (may be positive or negative)
##    - comment: comment on operation
##############################################################################
sub book_add {
    my $self = shift;
    my %test = @_;
    my %entry;

    %entry = map {$_ => undef} @book_entry_keys;
    lock_keys(%entry, @book_entry_keys);
    foreach my $key (keys %test) {
	$entry{$key} = $test{$key} if exists $entry{$key}
    }
    my $book = $self->{book};
    push @$book, \%entry;

    # Update calc_nb
    $self->{calc_nb} += $entry{number};
}


##############################################################################
## Delete last entry
## $wine->book_del
##############################################################################
sub book_del {
    my $self = shift;
    my $h = pop @{$self->{book}};
    # Update calc_nb
    $self->{calc_nb} += $h->{number};
}


##############################################################################
## Modify an entry
## $wine->book_modify(entry_ind, key1 => value1,... keyn => valuen)
##    entry_ind     : entry index in book
##    keyx => valuex: couple (key, value) to modify in book entry
##############################################################################
sub book_modify {
    my $self = shift;
    my $entry_ind = shift;
    my %values = @_;

    my $book = $self->{book};
    my $entry = $book->[$entry_ind];
    foreach (keys %values) {
	$entry->{$_} = $values{$_} if exists $entry->{$_}
     }
}


##############################################################################
## Return list of permitted keys for a book entry
##############################################################################
sub book_entry_get_keys {
    my $that = shift;
    return @book_entry_keys
}


##############################################################################
## Return list of book entries
## An entry is a hash ref. Possible keys for this hash are returned by 
## book_entry_get_keys
##############################################################################
sub book_get_entries {
    my $self = shift;
    return @{$self->{book}}
}



1;

__END__
