#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <gwine@free.fr>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <gwine@free.fr>
#############################################################################
## Define Gdb class
#############################################################################
## This class defines Gwine Database
#############################################################################

package Gwine::Gdb;
use warnings;
use strict;

use Gwine::GLAppel;
use Gwine::GLProd;
use Gwine::GLWine;
use Gwine::GLZone;
use Gwine::L10N;

use open ':utf8'; # Use UTF-8 layer for all giles

## Version of database
use constant DB_VERSION => 5;



#############################################################################
## Creator
## Parameters :
##    none
#############################################################################
sub new {
    my $proto = shift;

    my $class = ref($proto) || $proto;
    my $self = {};
    bless($self,$class);
    
    $self->clear;
    return $self;
}


#############################################################################
## Method to clear db
#############################################################################
sub clear {
    my $self = shift;

    $self->{zone}  = Gwine::GLZone->new();
    $self->{appel} = Gwine::GLAppel->new();
    $self->{prod}  = Gwine::GLProd->new();
    $self->{wine}  = Gwine::GLWine->new();
    return $self;
}

#############################################################################
## Method to get appellation db item
## Return a reference on db item
#############################################################################
sub get_appel {
  my $self = shift;
  my $appel = shift;

  my $list = $self->{appel};
  if ($list->size == 0) { return -1 }
  return $list->[$appel];
}


#############################################################################
## Method to get producer db item
## Return a reference on db item
#############################################################################
sub get_prod {
  my $self = shift;
  my $prod = shift;

  my $list = $self->{prod};
  if ($list->size == 0) { return -1 }
  if ($prod == -1) {return -1}
  return $list->[$prod];
}


#############################################################################
## Method to get wine db item
## Return a reference on db item
#############################################################################
sub get_wine {
  my $self = shift;
  my $wine = shift;

  my $list = $self->{wine};
  if ($list->size == 0) { return -1 }
  return $list->[$wine];
}


#############################################################################
## Method to get zone db item
## Return a reference on db item
#############################################################################
sub get_zone {
  my $self = shift;
  my $zone = shift;

  my $list = $self->{zone};
  if ($list->size == 0) { return -1 }
  return $list->[$zone];
}


#############################################################################
## Method to get zone path for a wine item
## Return a string
#############################################################################
sub get_zone_path {
    my $self = shift;
    my $wine_id = shift;

    my $appel = $self->wine->by_id($wine_id)->appellation;
    my ($app, $ext) = split /:/, $appel;
    my $path = $self->appel->by_id($app)->appellation;
    $path .= '/' . $self->appel->by_id($app)->get_extension($ext) 
	if defined $ext and $ext ne '';

    my $zone = $self->zone->by_id($self->appel->by_id($app)->vineyard);
    while ($zone != -1) {
	$path = $zone->zone . "/$path";
	$zone = $self->zone->by_id($zone->parent);
    }
    return $path
}


#############################################################################
## Is_db Method
## Verify if a file is a gwine db
## Parameter :
##    filename : string : absolute file name
## Return : a list containing file handler and version if filename is a gwine db
#############################################################################
## Be careful : this method read the first two lines from filename
## Support format version 1 and 2.
#############################################################################
sub is_db {
    my $self = shift;
    my $filename = shift;
    my ($fh, $version);

    if (! open($fh, "<$filename")) {
	warn "ERROR: cannot open file: $filename\n";
	return ();
    }
    
    # Verify first two lines of db file
    # First line must be "[info]"
    my $line = <$fh>;
    chomp $line;
    if ($line !~ m!^\[info\]!) {
	warn "ERROR: uncorrect database, first line in file is not : [info]\n";
	print STDERR "First line is $line\n";
	return ();
    }
    # Second line must start with "version="
    $line = <$fh>;
    chomp $line;
    if ($line !~ /^version=(\d+)/) {
	warn "ERROR: cannot find database version\n";
	return ();
    }
    else { $version = $1 }

    return ($fh, $version);
}

#############################################################################
## Open Methods
## Open a db from disk
## Parameter :
##    filename : string : absolute file name
## Return : -1 Ok
##           0 Errors occured
##          >0 Number of warnings  
#############################################################################
sub open_db {
    my $warn = 0; # Nb warning. -1 if no warning
    my ($fh, $version) = is_db(@_);
    my $self = shift;
    shift;
    my $lh = shift;

    if (! defined $version) { return 0 }

    if ($version < 1 or $version > DB_VERSION) {
	warn "Unknown database format version $version\n";
	return 0;
    }
    if ($version < 2) { warn "Old database format: some data may be lost\n" }


    if (!defined $fh) {	return $warn }
    # Here begin analysis of database
    my $id = -1;
    my $type = "";
    while (<$fh>) {
	chomp;
	if (m!^\[wine (\d+)\]!) {
	    $id = $1;
	    $type = 'wine';
	    my $ref = Gwine::GWine->new;
	    $self->{wine}->by_id($id, $ref);
	    next;
	}
	elsif (m!^\[vineyard (\d+)\]!) { # V1 only
	    $id = $1;
	    $type = 'vine';
	    my $ref = Gwine::GZone->new;
	    $self->{zone}->by_id($id, $ref);
	    next;
	}
	elsif (m!^\[zone (\d+)\]!) { # V2 only
	    $id = $1;
	    $type = 'zone';
	    my $ref = Gwine::GZone->new;
	    $self->{zone}->by_id($id, $ref);
	    next;
	}
	elsif (m!^\[produc(er|tor) (\d+)\]!) {
	    $id = $2;
	    $type = 'prod';
	    my $ref = Gwine::GProd->new;
	    $self->{prod}->by_id($id, $ref);
	    next;
	}
	elsif (m!^\[appellation (\d+)\]!) {
	    $id = $1;
	    $type = 'appel';
	    my $ref = Gwine::GAppel->new;
	    $self->{appel}->by_id($id, $ref);
	    next;
	}

	if ($type eq 'vine') {
	  SWITCH_VINE: {
	      # Now, vineyards are transformed to zones.
	      if (m!^vineyard=(.+)!) { $self->{zone}->by_id($id)->zone($1); 
				       $self->{zone}->by_id($id)->parent(-1); last }
	      if ($_ !~ m!^$!) { warn "Database, line $. : unknown value\n"; $warn++ }
	  }
	}
	if ($type eq 'zone') {
	  SWITCH_ZONE: {
	      if (m!^zone=(.+)!) { $self->{zone}->by_id($id)->zone($1); last }
	      if (m!^parent=(-?\d+)!) {$self->{zone}->by_id($id)->parent($1); last }
	      if ($_ !~ m!^$!) { warn "Database, line $. : unknown value\n"; $warn++ }
	  }
	}
	if ($type eq 'appel') {
	  SWITCH_APPEL: {
	      if (m!^appellation=(.+)!) { $self->{appel}->by_id($id)->appellation($1); last }
	      if (m!^vineyard=(\d+)!) { $self->{appel}->by_id($id)->vineyard($1); last }
	      if (m!^villages=(.*)!) {
		  $self->{appel}->by_id($id)->extensions( [split(/,/, $1)] ); last } # Db V1
	      if (m!^extensions=(.*)!) {
		  $self->{appel}->by_id($id)->extensions( [split(/,/, $1)] ); last } # Db V2
	      if ($_ !~ m!^$!) { warn "Database, line $. : unknown value\n"; $warn++ }
	  }
	}
	if ($type eq 'prod') {
	  SWITCH_PROD: {
	      if (m!^name=(.+)!) { $self->{prod}->by_id($id)->name($1); last }
	      if (m!^address=(.*)!) { $self->{prod}->by_id($id)->address($1); last }
	      if (m!^zip=(.*)!) { $self->{prod}->by_id($id)->zip($1); last }
	      if (m!^city=(.*)!) { $self->{prod}->by_id($id)->city($1); last }
	      if (m!^country=(.*)!) { $self->{prod}->by_id($id)->country($1); last }
	      if (m!^phone=(.*)!) { $self->{prod}->by_id($id)->phone($1); last }
	      if ($_ !~ m!^$!) { warn "Database, line $. : unknown value\n"; $warn++ }
	  }
	}
	if ($type eq 'wine') {
	  SWITCH_WINE: {
	      if (m!^domain=(.*)!) { my $t = $1; $self->{wine}->by_id($id)->domain($t); last }
	      if (m!^millesime=(\d+)!) { my $t = $1; $self->{wine}->by_id($id)->millesime($t); last }
	      if (m!^color=(\d+)!) { my $t = $1; $self->{wine}->by_id($id)->color($t); last }
	      if (m!^bdate=(.+)!) { 
		  my $date = $1;		  
		  # If $date is only a year (for DB version < 4), convert it to year-01-01
		  if (length($date) < 10) { $self->{wine}->by_id($id)->date("$date-01-01") }
		  else { $self->{wine}->by_id($id)->date($lh->save2date($date)) }
		  last 
	      }
	      if (m!^bquantity=(\d+)!) { my $t = $1; $self->{wine}->by_id($id)->number($t); last }
	      if (m!^bsize=(\d+(\.\d+)?)!) { my $t = $1; $self->{wine}->by_id($id)->size($lh->numf($t)); last }
	      if (m!^price=(\d+(\.\d+)?)!) { my $t = $1; $self->{wine}->by_id($id)->price($lh->numf($t)); last }
	      if (m!^appellation=(\d+:\d*)!) { my $t = $1; $self->{wine}->by_id($id)->appellation($t); last }
	      if (m!^produc(er|tor)=(\d*)!) { my $t = $2; $self->{wine}->by_id($id)->producer(($t eq "") ? -1 : $t); last }
	      if (m!^conservation=(\d*)!) { my $t = $1; $self->{wine}->by_id($id)->conservation($t); last }
	      if (m!^notes=(.*)!) { 
		  my $notes = $1;
		  $notes =~ s|\\n|\n|g; # Regenerate "\n"
		  $self->{wine}->by_id($id)->notes($notes); 
		  last 
		  }
	      if (m!^book_entry(\d+)=(.*)!) {
		  my ($i, $line) = ($1, $2);
		  my ($date, $number, @comment) = split /:/, $line;
		  my %entry;
		  $entry{date} = $lh->save2date($date);
		  $entry{number} = $number;
		  $entry{comment} = join ',', @comment;	
		  $self->{wine}->by_id($id)->book_add(%entry);
		  last
	      }
	      if ($_ !~ m!^$!) { warn "Database, line $. : unknown value\n"; $warn++ }
	  }
	}
    }

    close $fh;
    
    # Verify there is no hole in lists
    my $size = $self->zone->size;
    for(my $i = 0; $i < $size; $i++) {
	if (!defined $self->zone->by_id($i)) { $self->zone->by_id($i, -1) }
    }
    $size = $self->appel->size;
    for(my $i = 0; $i < $size; $i++) {
	if (!defined $self->appel->by_id($i)) { $self->appel->by_id($i, -1) }
    }
    $size = $self->wine->size;
    for(my $i = 0; $i < $size; $i++) {
	if (!defined $self->wine->by_id($i)) { $self->wine->by_id($i, -1) }
	# in the same time, data field in wine object is felt with zone path
	else { $self->wine->by_id($i)->data($self->get_zone_path($i))	}
    }
    $size = $self->prod->size;
    for(my $i = 0; $i < $size; $i++) {
	if (!defined $self->prod->by_id($i)) { $self->prod->by_id($i, -1) }
    }

    if ($warn) { return $warn }
    else { return -1 };
}

#############################################################################
## Save Methods.
## Save a db on disk
## Parameter :
##    filename : string : absolute file name
##    $fh : ref : file handler
##    $lh : ref : language handler
#############################################################################
#############################################################################
## This functions write items to $fh
#############################################################################
sub _save_wine {
    my $self = shift;
    my $wine = shift;
    my $fh = shift;
    my $lh = shift;

    if ($wine ne -1) {
	my $domain = $wine->domain;
	my $appel = $wine->appellation;
	my $millesime = $wine->millesime;
	my $color = $wine->color;
	my $date = $lh->date2save($wine->date);
	my $quantity = $wine->number;
	my $size = $lh->numunf($wine->size);
	# Saved with decimal point to ".". Thus, in other locale,
	# file saved may be read
	my $price = $lh->numunf($wine->price);
	my $prod = $wine->producer;
	my $cons = $wine->conservation;
	my $notes = $wine->notes;
	my @book = $wine->book_get_entries;
	# Replace all "\n" by '\n' to have only one line in 
	# notes field
	$notes =~ s|\n|\\n|g;
	
	my $i = $self->wine->get_id($wine);

	# Save only fields containing values
	print $fh "[wine $i]\n";
	print $fh "domain=$domain\n" if $domain;
	print $fh "appellation=$appel\n";
	print $fh "millesime=$millesime\n" if $millesime;
	print $fh "color=$color\n" if defined $color;
	print $fh "bdate=$date\n" if $date;
	print $fh "bquantity=$quantity\n" if $quantity;
	print $fh "bsize=$size\n" if $size;
	print $fh "price=$price\n" if $price;
	print $fh "conservation=$cons\n" if $cons;
	print $fh "producer=$prod\n" if $prod;
	print $fh "notes=$notes\n" if $notes;
	$i = 0;
	foreach my $entry (@book) {
	    print $fh "book_entry$i=";
	    print $fh join(':', $lh->date2save($entry->{date}), 
			   $entry->{number}, $entry->{comment}), "\n";
	    $i++;
	}
	print $fh "\n";
	return 1
    }
    return 0
}


sub _save_appel {
    my $self = shift;
    my $appel = shift;
    my $fh = shift;
    
    if ($appel ne "-1") {
	my $appellation = $appel->appellation;
	my $vine = $appel->vineyard;
	my $extensions = join(',', @{$appel->extensions});
	
	my $i = $self->appel->get_id($appel);

	print $fh "[appellation $i]\n";
	print $fh "appellation=$appellation\n";
	print $fh "extensions=$extensions\n" if @{$appel->extensions};
	print $fh "vineyard=$vine\n\n";
	return 1
    }
    return 0
}


sub _save_prod {
    my $self = shift;
    my $prod = shift;
    my $fh = shift;

    if ($prod ne "-1") {
	my $name = $prod->name;
	my $address = $prod->address;
	my $zip = $prod->zip;
	my $city = $prod->city;
	my $country = $prod->country;
	my $phone = $prod->phone;
	
	my $i = $self->prod->get_id($prod);

	print $fh "[producer $i]\n";
	print $fh "name=$name\n" if $name;
	print $fh "address=$address\n" if $address;
	print $fh "zip=$zip\n" if $zip;
	print $fh "city=$city\n" if $city;
	print $fh "country=$country\n" if $country;
	print $fh "phone=$phone\n" if $phone;
	print $fh "\n";
	return 1
    }
    return 0
}


sub _save_zone {
    my $self = shift;
    my $zone = shift;
    my $fh = shift;
    
    if ($zone ne "-1") {
	my $geo_zone = $zone->zone;
	my $parent = $zone->parent;
	
	my $i = $self->zone->get_id($zone);

	print $fh "[zone $i]\n";
	print $fh "zone=$geo_zone\n";
	print $fh "parent=$parent\n\n";
	return 1
    }
    return 0
}


my %next_step;

#############################################################################
## Save db step by step. Must be called for first step.
#############################################################################
sub save_by_step {
    my $self = shift;
    my $filename = shift;
    my $lh = shift;

    open (my $fh, ">$filename") or do {warn "ERROR: cannor open $filename"; return 0};
    my $time = localtime;
    print $fh "[info]\n";
    print $fh "version=" . DB_VERSION . "\n";
    print $fh "type_db=all\n";
    print $fh "nb_zone=" . $self->zone->true_size . "\n";
    print $fh "nb_appellation=" . $self->appel->true_size . "\n";
    print $fh "nb_wine=" . $self->wine->true_size . "\n";
    print $fh "nb_prod=" . $self->prod->true_size . "\n";
    print $fh "date=$time\n\n\n";
    %next_step = (type => 'wine', idx => 0, fh => $fh, lh => $lh);
    return 1;
}


#############################################################################
## Save next item. Before calling this procedure first, you must call 
## save_by_step.
## Return :
##   0 : no more item to save
##   1 : saving wines
##   2 : saving appellations
##   3 : saving zones
##   4 : saving producers
##  -1 : this step, nothing was saved
#############################################################################
sub save_next_step {
    my $self = shift;

    my $idx = $next_step{idx};
    
    if ($next_step{type} eq 'wine') {
	if ($next_step{idx} < $self->wine->size) {
	    return -1 if ! _save_wine($self, $self->wine->by_id($next_step{idx}++), 
				    $next_step{fh}, $next_step{lh});
	}
	else {
	    $next_step{type} = 'appel';
	    $next_step{idx} = 0;
	    return -1
	}
	return 1
    }
    elsif ($next_step{type} eq 'appel') {
	if ($next_step{idx} < $self->appel->size) {
	    return -1 if !_save_appel($self, $self->appel->by_id($next_step{idx}++), 
				      $next_step{fh}, $next_step{lh});
	}
	else {
	    $next_step{type} = 'zone';
	    $next_step{idx} = 0;
	    return -1
	}
	return 2
    }
    elsif ($next_step{type} eq 'zone') {
	if ($next_step{idx} < $self->zone->size) {
	    return -1 if !_save_zone($self, $self->zone->by_id($next_step{idx}++), 
		       $next_step{fh}, $next_step{lh});
	}
	else {
	    $next_step{type} = 'prod';
	    $next_step{idx} = 0;
	    return -1
	}
	return 3
    }
    elsif ($next_step{type} eq 'prod') {
	if ($next_step{idx} < $self->prod->size) {
	    return -1 if !_save_prod($self, $self->prod->by_id($next_step{idx}++), 
		       $next_step{fh}, $next_step{lh});
	}
	else { 
	    %next_step = (type => 'undef') ;
	    return -1
}
	return 4
    }
    return 0
}

#############################################################################
## Save all db in one step.
#############################################################################
sub save_db {
  my $self = shift;
  my $filename = shift;
  my $lh = shift;

  open (my $fh, ">$filename") or do {warn "ERROR: cannor open $filename"; return 0};
  my $time = localtime;
  print $fh "[info]\n";
  print $fh "version=" . DB_VERSION . "\n";
  print $fh "type_db=all\n";
  print $fh "nb_zone=" . $self->zone->true_size . "\n";
  print $fh "nb_appellation=" . $self->appel->true_size . "\n";
  print $fh "nb_wine=" . $self->wine->true_size . "\n";
  print $fh "nb_prod=" . $self->prod->true_size . "\n";
  print $fh "date=$time\n\n\n";

  _save_wine($self, $_, $fh, $lh) foreach $self->wine->list;
  _save_appel($self, $_, $fh, $lh) foreach $self->appel->list;
  _save_zone($self, $_, $fh, $lh) foreach $self->zone->list;
  _save_prod($self, $_, $fh, $lh) foreach $self->prod->list;
  return 1;
}


#############################################################################
## Methods to get lists of vineyards, appellations, producers and wines names
## Return a list of strings
#############################################################################
sub list_zone {
  my $self = shift;
  my @list = $self->{zone}->list();
  my @list_zone;
  foreach (@list) {
      if ($_ ne -1) { push @list_zone, $_->zone() }
  }
  return @list_zone;
}

sub list_appel {
  my $self = shift;
  my @list = $self->{appel}->list();
  my @list_appel;
  foreach (@list) {
      if ($_ ne -1) { push @list_appel, $_->appellation() }
      }
  return @list_appel;
}

sub list_prod {
  my $self = shift;
  my @list = $self->{prod}->list();
  my @list_prod;
  foreach (@list) {
      if ($_ ne -1) { push @list_prod, $_->name() }
  }
  return @list_prod;
}

sub list_wine {
  my $self = shift;
  my @list = $self->{wine}->list();
  my @list_wine;
  foreach (@list) {
      if ($_ ne -1) { push @list_wine, $_->domain() }
  }
  return @list_wine;
}


#############################################################################
## Methods to access per-object data
## With args, they set the value, else they only retrieve it.
#############################################################################
sub zone {
  my $self = shift;
  if (@_)  {$self->{zone} = shift}
  return $self->{zone};
}

sub appel {
  my $self = shift;
  if (@_)  {$self->{appel} = shift}
  return $self->{appel};
}

sub prod {
  my $self = shift;
  if (@_)  {$self->{prod} = shift}
  return $self->{prod};
}

sub wine {
  my $self = shift;
  if (@_)  {$self->{wine} = shift}
  return $self->{wine};
}




1;

__END__
