#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <gwine@free.fr>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <gwine@free.fr>
#############################################################################
use strict;
use warnings;

####################################################################
## Package Gwine::L10N
## Base class for all L10N classes
####################################################################

package Gwine::L10N;

use I18N::LangTags;
use DateTime;

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = bless {}, $class;
    $self->init;
    return $self
}


#############################################################################
## get_l10n (this is a constructor)
sub get_l10n {
    my $class = shift;

    my @languages;
    if(length( $ENV{'LANG'} || '' )) {
	push @languages, $ENV{'LANG'};
    }
    if(length( $ENV{'LANGUAGE'} || '' )) {
	push @languages, $ENV{'LANGUAGE'};
    }
    
    @languages = $class->_get_langtag(@languages);

    my %seen;
    my @pm = map { $class . "::" . $_ }  @languages;
  foreach my $module ( @pm ) {
      next unless length $module;
      next if $seen{$module}++;
      { 
	  local $SIG{'__DIE__'};
	  eval "require $module";
      }
      return $module->new if !$@;
  }
    return $class->new
}

#############################################################################
## called new
sub init {
    my $self = shift;

    # Create a DateTime object
    use POSIX qw( locale_h );
    my $locale = setlocale(LC_TIME);
    no POSIX;
    # DateTime::Locale uses aa_AA_variant format
    # But, on linux, locales use aa_AA[.charset][@variant]
    # First, suppress charset
    $locale =~ s/\.[\w\d-]+//g;
    # Then, change @euro variant to PREEURO
    $locale =~ s/\@euro/_PREEURO/g;
    $self->{datetime} = DateTime->new( year => 2003, locale => $locale );
    # Create a number formatter. 
    $self->{decimal_digits} = 2;
    $self->{decimal_fill} = 1;
    $self->{decimal_point} = '.';
}


#############################################################################
## Return datetime object
sub datetime { return $_[0]->{datetime} }


#############################################################################
## Format a number according to locale
## If $number is already formatting according to locale, just return it
sub numf {
    my $self = shift;
    my $number = shift;

    return 0 if $number eq '';
    
    my $dp = $self->{decimal_point};
    return $number if $number =~ /$dp/;

    my $format = '%.' . ($self->{decimal_fill} ? $self->{decimal_digits} : 0) 
	. 'f';
    $number = sprintf $format, $number;
    $number =~ s/\./$dp/;
    return $number;
}


#############################################################################
# Method to unformat a number.
sub numunf {
    my $self = shift;
    my $number = shift;
    
    my $dp = $self->{decimal_point};
    $dp =~ s/([\.\*\?\+])/\\$1/g;
    return $number if $number !~ /$dp/;

    $number =~ s/$dp/\./;
    return $number;
}


#############################################################################
# Method to format a date according to locale from save format (yyyy-mm-dd).
sub save2date {
    my $self = shift;
    my $save = shift;
    
    $save =~ m/(\d+)-(\d+)-(\d+)/;
    my ($year, $month, $day) = ($1, $2, $3);
    $self->{datetime}->set(year => $year, month => $month, day => $day);
    return $self->_to_locale_format;
}


#############################################################################
# Method to unformat a date from locale to save format (yyyy-mm-dd).
sub date2save {
    my $self = shift;
    my $date = shift;

    my @format = split //, $self->{datetime}->locale->date_parts_order;
    foreach my $f (@format) {
	if ($f eq 'y') {
	    $date =~ s/^(\d+)\D?//;
	    my $year = $1;
	    $self->{datetime}->set(year => $year);
	    next
	}
	if ($f eq 'm') {
	    $date =~ s/^(\d+)\D?//;
	    my $month = $1;
	    $self->{datetime}->set(month => $month);
	    next
	}
	if ($f eq 'd') {
	    $date =~ s/^(\d+)\D?//;
	    my $day = $1;
	    $self->{datetime}->set(day => $day);
	    next
	}
    }
    return $self->{datetime}->ymd('-')
}


#############################################################################
## Internal functions

sub _add_supers {
  my($class, @languages) = @_;

  my %h;
  $h{$_} = 1 foreach @languages;
  
  my(@output_languages);
  foreach my $lang (@languages) {
      push @output_languages, $lang;
      foreach my $s ( I18N::LangTags::super_languages($lang) ) {
	  last if $h{$s};
	  push @output_languages, $s;
      }
  }
  @languages = @output_languages;
  
  return @languages;
}

sub _get_langtag {
  my($class, @languages) = @_;

  @languages = map &I18N::LangTags::locale2language_tag($_), @languages;
  @languages = map {; $_, I18N::LangTags::alternate_language_tags($_) } @languages;
  
  @languages     = $class->_add_supers( @languages );
  
  @languages = map {
	  my $l = lc $_;
	  $l =~ tr/-/_/;
	  $l =~ s/[^a-z0-9_]//g;
	  $l;
      } @languages;

  return @languages;
}


#############################################################################
#
#                       Localizable Methods
#
#############################################################################


#############################################################################
# Method to format a date according to locale.
sub _to_locale_format {
    my $self = shift;
    return $self->{datetime}->strftime('%m-%d-%Y');
}


#############################################################################
## By-date sort function
sub sort_by_date {
    my ($lh, $a, $b) = @_;

    $a ||= '0-0-0';
    $b ||= '0-0-0';

    my ($ma, $da, $ya) = split /-/, $a;
    my ($mb, $db, $yb) = split /-/, $b;
    my $year = $ya <=> $yb;
    return $year if $year != 0;
    my $month = $ma <=> $mb;
    return $month if $month != 0;
    return ($da <=> $db)
}




1;


__END__
