#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <gwine@free.fr>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <gwine@free.fr>
#############################################################################
## This file contains handlers for gwine UI
#############################################################################
require 5.8.1;
use warnings;

#==============================================================================
#=== This is the 'AppGwine' class                              
#==============================================================================
package AppGwine;
use strict;

BEGIN {
    use Encode;
    use Data::Dumper;
    use File::Spec;
    use Gwine::Config;
    # UI
    use Gwine::DialogWine;
    use Gwine::DialogProd;
    use Gwine::DialogPref;
    use Gwine::DialogCellar;
}


use constant FALSE => 0;
use constant TRUE  => 1;


##################################################################
## Global variables
##################################################################
our ($config, $db, $gxml, %pixmaps, $VERSION, $lh);
my @wine_colors;
my @wine_cols;
my @prod_cols;
my ($wine_iter, $prod_iter);

$VERSION = '0.10.3';

sub wine_cols { @wine_cols }
sub prod_cols { @prod_cols }


##################################################################
## helper function
##################################################################
sub __u { decode('UTF-8', __($_[0])) }
sub __xu { decode('UTF-8', __x(@_)) }


##################################################################
## app_gwine init
##################################################################
sub init {
    my $class = shift;

    # Create an object for some Localization features
    $lh = Gwine::L10N->get_l10n();

    # Arrays are initialized here because __ cannot be called outside of sub
    @wine_cols = (__u('Domain'), __u('Vintage'), __u('Color'), __u('Number'),
		  __u('Price'), __u('Buying date'), __u('Size'), __u('Producer'),
		  __u('Zone'), 'Wine_id');
    my @wine_col_types = ('Glib::String', 'Glib::String', 'Glib::String', 'Glib::Int', 
			  'Glib::String', 'Glib::String', 'Glib::String', 'Glib::String',
			  'Glib::String', 'Glib::String');
    @prod_cols = (__u('Name'), __u('Address'), __u('Zip Code'), __u('City'), 
		  __u('Country'), __u('Phone'), 'Prod_id');
    my @prod_col_types = ('Glib::String', 'Glib::String', 'Glib::String',
		       'Glib::String', 'Glib::String', 'Glib::String', 'Glib::Int');

    # Init wine colors
    @wine_colors = (__u('White'), __u('Rose'), __u('Red'));

    # Init Zone Treeview
    my $view = $gxml->get_widget('ztreeview');
    my $ztstore = Gtk2::TreeStore->new('Gtk2::Gdk::Pixbuf', 'Glib::String', 
						'Glib::String');
    $ztstore->set_default_sort_func("$class\::ztree_sort");
    $ztstore->set_sort_column_id(1, 'ascending');
    $ztstore->set_sort_func(1, "$class\::ztree_sort");
    $view->set_model($ztstore);
    my $renderer = Gtk2::CellRendererPixbuf->new;
    my $col = Gtk2::TreeViewColumn->new_with_attributes('', $renderer, 'pixbuf', 0);
    $view->append_column($col);
    $renderer = Gtk2::CellRendererText->new;
    $renderer->set('editable', TRUE);
    $renderer->signal_connect('edited', "$class\::on_renderer_text_edited_event");
    $col = Gtk2::TreeViewColumn->new_with_attributes('', $renderer, 'text', 1);
    $view->append_column($col);
    $view->get_selection->set_mode('single');
    $view->get_selection->signal_connect('changed', "$class\::on_ztreeview_selection_changed");

    # Init Wine Treeview
    $view = $gxml->get_widget('wtreeview');
    my $wlstore = Gtk2::ListStore->new(@wine_col_types);
    $view->set_model($wlstore);
    # add all columns but last to View : last column of model is only 
    # for internal purpose
    for (my $i = 0; $i < @wine_cols-1; $i++) {
	my $renderer = Gtk2::CellRendererText->new;
	my $col = Gtk2::TreeViewColumn->new_with_attributes($wine_cols[$i],
							    $renderer,
							    'text', $i);
	$col->set_clickable(TRUE);
	$col->signal_connect('clicked', "$class\::on_wtreeview_column_clicked", $i);
	$col->set_resizable(TRUE);
	$view->append_column($col);
	$wlstore->set_sort_func($i, "$class\::wlist_sort");
    }
    my $select = $view->get_selection;
    $select->set_mode('single');
    $select->signal_connect('changed', "$class\::on_wtreeview_selection_changed");

    # Init Producers Treeview
    $view = $gxml->get_widget('ptreeview');
    my $plstore = Gtk2::ListStore->new(@prod_col_types);
    $plstore->set_default_sort_func("$class\::plist_sort");
    $plstore->set_sort_column_id(0, 'ascending');
    $view->set_model($plstore);
    # add all columns but last to View : last column of model is only 
    # for internal purpose
    for (my $i = 0; $i < @prod_cols-1; $i++) {
	my $renderer = Gtk2::CellRendererText->new;
	my $col = Gtk2::TreeViewColumn->new_with_attributes($prod_cols[$i], 
							    $renderer,
							    'text', $i);
	$col->set_clickable(TRUE);
	$col->signal_connect('clicked', "$class\::on_ptreeview_column_clicked", $i);
	$col->set_resizable(TRUE);
	$view->append_column($col);
    }
    $select = $view->get_selection;
    $select->set_mode('single');
    $select->signal_connect('changed', "$class\::on_ptreeview_selection_changed");
}


# Set visibility for wine list columns
sub wine_list_columns_set_visible {
    my @visibility = @_;
    my @columns = $gxml->get_widget('wtreeview')->get_columns;
    foreach my $col (@columns) { $col->set_visible(shift @visibility) if @visibility }
}


# Get visibility for wine list columns
sub wine_list_columns_get_visible {
    my @columns = $gxml->get_widget('wtreeview')->get_columns;
    return map $_->get_visible, @columns
}


# Set visibility for prod list columns
sub prod_list_columns_set_visible {
    my @visibility = @_;
    my @columns = $gxml->get_widget('ptreeview')->get_columns;
    foreach my $col (@columns) { $col->set_visible(shift @visibility) if @visibility }
}


# Get visibility for prod list columns
sub prod_list_columns_get_visible {
    my @columns = $gxml->get_widget('ptreeview')->get_columns;
    return map $_->get_visible, @columns
}


##################################################################
## File menu
##################################################################
sub on_new_event {
    my $dialog = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),
					  'destroy-with-parent',
					  'question',
					  'none',
					  __xu("Save {file} before creating new cellar? All unsaved data will be lost.", 
					      file => $config->opened_file));
    $dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9, 'gtk-cancel', -6);
    my $return = $dialog->run;
    $dialog->destroy;
    return if $return eq 'cancel';
    # Save db if 'yes'
    on_save_event() if $return eq 'yes';
    
    # Delete all db in memory
    $db->clear;
    my $name = $config->opened_file(__u('New Cellar'));
    $gxml->get_widget('app_gwine')->set_title(__xu("Gwine - {filename}", 
						  filename => $name));
    # Clear Tree
    $gxml->get_widget('ztreeview')->signal_emit('map');
    # Clear Lists
    $gxml->get_widget('wtreeview')->get_model->clear;
    $gxml->get_widget('ptreeview')->get_model->clear;
}

sub open_file {
    my $filename = shift;
    $gxml->get_widget('appbar')->set_status(__xu('Opening {file}...', 
						file => $filename));
    $db->clear;
    if ($db->open_db($filename, $lh)) {
	$config->opened_file($filename);
	$gxml->get_widget('app_gwine')->set_title("$filename");
	$gxml->get_widget('ztreeview')->signal_emit('map');
	$gxml->get_widget('appbar')->set_status('');
	return 1
    }
    # Errors detected while opening $filename
    return 0
}

sub on_open_event {
    my $filesel = Gtk2::FileChooserDialog->new(__u("Open file..."),
					       $gxml->get_widget('app_gwine'),
					       'open', 'gtk-cancel', 'cancel',
					       'gtk-ok', 'ok');
    my $resp = $filesel->run;
    my $filename = $filesel->get_filename if $resp eq 'ok';
    $filesel->destroy;
    return if !defined $filename;
    open_file($filename);
    # Select Wines node to refresh list view
    my $select = $gxml->get_widget('ztreeview')->get_selection;
    $select->select_iter($wine_iter)
}

sub on_save_event {
    if (defined $config->opened_file and $config->opened_file ne __u('New Cellar')) {
	$db->save_by_step($config->opened_file, $lh);
	my $total = $db->wine->true_size + $db->appel->true_size
	    + $db->zone->true_size + $db->prod->true_size;
	my $i = 1;
	my $appbar = $gxml->get_widget('appbar');
	while (my $res = $db->save_next_step) {
	    next if $res == -1;
	    $appbar->set_progress_percentage($i / $total) if $total;
	    $appbar->pop;
	    $appbar->push(__xu('saving... ({i}/{total})', 
			     i => $i, total => $total));
	    $i++;
	    Gtk2->main_iteration;
	}
	$appbar->set_status(__xu('{file} saved', file => $config->opened_file));
    }
    else { on_save_as_event(@_) }
}

sub on_save_as_event {
    my $filesel = Gtk2::FileChooserDialog->new(__u("Save file as..."),
					       $gxml->get_widget('app_gwine'),
					       'save', 'gtk-cancel', 'cancel',
					       'gtk-ok', 'ok');
    $filesel->set_filename($config->opened_file) if defined $config->opened_file;
    my $resp = $filesel->run;
    if ($resp eq 'ok') {
	my $filename = $filesel->get_filename;
	$db->save_by_step($filename, $lh);
	my $total = $db->wine->true_size + $db->appel->true_size
	    + $db->zone->true_size + $db->prod->true_size;
	my $i = 0;
	my $appbar = $gxml->get_widget('appbar');
	while (my $res = $db->save_next_step) {
	    next if $res == -1;
	    $i++;
	    $appbar->set_progress_percentage($i / $total) if $total;
	    $appbar->pop;
	    $appbar->push(__xu('saving {file}... ({i}/{total})', 
			      file => $filename, i => $i, 
			      total => $total));
	    Gtk2->main_iteration;
	}
	$config->opened_file($filename);
	$gxml->get_widget('app_gwine')->set_title("Gwine - $filename");
	$appbar->set_status(__xu('{file} saved', file => $config->opened_file));
    }
    $filesel->destroy;
}

sub on_quit_event {
    if ($config->conf_popups) {
	# Popus Dialog "Yes/No"
	my $dialog = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),
					      'destroy-with-parent',
					      'question',
					      'none',
					      __u('All unsaved data will be lost. Do you want to quit?'));
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Save config
    my ($width, $heigth) = $gxml->get_widget('app_gwine')->get_size;
    $config->geometry("${width}x$heigth");
    $config->paned_pos($gxml->get_widget('hpaned')->get_position);
    # This may be do only in on_preferences_event sub
#    my @columns = $gxml->get_widget('wtreeview')->get_columns;
#    my @visible = map $_->get_visible, @columns;
#    $config->wine_visible(\@visible);
#    @columns = $gxml->get_widget('ptreeview')->get_columns;
#    @visible = map $_->get_visible, @columns;
#    $config->prod_visible(\@visible);
    $config->save;

    Gtk2->main_quit
}


###################################################################
# Edit menu
###################################################################
sub on_properties_event {
    my $wine_quantity = $db->wine->true_size;
    my $bottle_quantity = 0;
    my @wines = $db->wine->list;
    foreach my $wine (@wines) {
	next if !defined $wine;
	next if $wine eq '-1';
	$bottle_quantity += $wine->real_number;
    }
    my (undef, $loc, $file)  = File::Spec->splitpath($config->opened_file);
    my ($size, $date) = (stat($config->opened_file))[7,9];

    # If size >= 2k, then size is written in kB (rounded to 1 digit 
    # after decimal point)
    if ($size > 2047) {
	$size = sprintf("%.1f", $size / 1024);
	$size =__xu('{size}k', size => $size);
    }

    my $dt = DateTime->from_epoch(epoch => $date, 
				  locale => $lh->datetime->locale);
    my $format = $dt->locale->long_date_format . ' ' 
	. $dt->locale->short_time_format;
    $date = $dt->strftime($format);

    my $path = $config->xml_path;
    my $xml = Gtk2::GladeXML->new("$path/gwine.glade", 'dialog_prop',
				  'gwine');
    $path = $config->icon_path;
    $xml->get_widget('dialog_prop')->set_icon_from_file("$path/gwine.png");
    $xml->get_widget('pix_name')->set_from_file("$path/gwine.png");
    $xml->get_widget('entry_name')->append_text($file);
    $xml->get_widget('label_size2')->set_text($size);
    $xml->get_widget('label_loc2')->set_text($loc);
    $xml->get_widget('label_date2')->set_text($date);
    $xml->get_widget('label_wquant')->set_text($wine_quantity);
    $xml->get_widget('label_bquant')->set_text($bottle_quantity);
    my $dialog = $xml->get_widget('dialog_prop');
    $dialog->signal_connect( response => sub { $_[0]->destroy } );
    $dialog->signal_connect( 'delete-event' => sub { $_[0]->destroy } );
}

sub on_preferences_event {
    my $dp = DialogPref->create($config);
    my $path = $config->icon_path;
    $dp->xml->get_widget('dialog_pref')->set_icon_from_file("$path/gwine.png");
}


###################################################################
# View menu
###################################################################
sub on_view_radio_event {
    my $radio = shift;

    my $active = $radio->get_active;
    if ($active) {
	my $group = $radio->get_group;
	my $i = 0;
	foreach (reverse @$group) {
	    if ($radio == $_ and $i == 0) {
		# Show wine list and hide tree
		my $select = $gxml->get_widget('ztreeview')->get_selection;
		$select->select_path(Gtk2::TreePath->new("0"));
		$gxml->get_widget('vbox1')->hide;
		last
	    }
	    elsif ($radio == $_ and $i == 1) {
		# Show prod list and hide tree
		my $select = $gxml->get_widget('ztreeview')->get_selection;
		$select->select_path(Gtk2::TreePath->new("1"));
		$gxml->get_widget('vbox1')->hide;
		last
	    }
	    elsif ($radio == $_ and $i == 2) {
		# Show tree
		$gxml->get_widget('vbox1')->show_all;
		last
	    }
	    $i++;
	}
	$config->view_style($i)
    }
}


sub on_view_cellar_book_event {    
    my $dc = DialogCellar->create($config->xml_path);
    my $path = $config->icon_path;
    $dc->xml->get_widget('dialog_cellar')->set_icon_from_file("$path/gwine.png");
    foreach my $wine ($db->wine->list) {
	next if $wine eq '-1';
	# First, add first entry
	my ($appel, $ext) = split /:/, $wine->appellation;
	my $appname = $db->appel->by_id($appel)->appellation;
	$appname .= ' ' . $db->appel->by_id($appel)->get_extension($ext)
	    if (defined $ext) and ($ext ne '');
	my $nb = $wine->number;
	$dc->add_entry("+$nb", $wine->date, $appname, $wine->domain, 
		       $wine->millesime, $wine->price, '');
	# Then add, each entry from cellar book
	my @entries = $wine->book_get_entries;
	$dc->add_entry(($_->{number} > 0 ? '+' : '') . $_->{number}, 
		       $_->{date}, $appname, $wine->domain, 
		       $wine->millesime, '', $_->{comment})
	    foreach @entries;
	
    }
}


sub on_view_refresh_event {
    my ($model, $iter) = $gxml->get_widget('ztreeview')->get_selection->get_selected;
    my $path = $model->get_path($iter)->to_string;;
    if ($path eq '1') { list_prods() }
    else { list_wines() }
    return 1
}


###################################################################
# Help menu
##################################################################
sub on_contents_activate { Gnome2::Help->display("gwine.xml") }


sub on_about_event {
    my $path = $config->xml_path;
    my $axml = Gtk2::GladeXML->new("$path/gwine.glade", 'about_gwine');
    $path = $config->icon_path;
    my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file("$path/gwine.png");
    $axml->get_widget('about_gwine')->set_logo($pixbuf);
    $axml->get_widget('about_gwine')->set_icon($pixbuf);
    $axml->get_widget('about_gwine')->set_translator_credits(__u('Sylvain Daubert <gwine@free.fr>'));
    $axml->get_widget('about_gwine')->set_version($VERSION);
    $axml->get_widget('about_gwine')->set_comments(__u("A wine cellar manager"))
}


###################################################################
# Gwine/Zones menu
###################################################################
sub on_new_zone_zone_event { on_new_zone_event($_[0], 0) }
sub on_new_appellation_zone_event { on_new_zone_event($_[0], 1) }
sub on_new_extension_zone_event { on_new_zone_event($_[0], 2) }

sub on_new_zone_event {
    my (undef, $action) = @_;
    my $view = $gxml->get_widget('ztreeview');
    my $store = $view->get_model;
    my (undef, $iparent) = $view->get_selection->get_selected;
    my ($parent_type, $parent_id) = split m!/!, $store->get_value($iparent, 2);
    
    # Parent is root...
    $parent_id = -1 if $parent_type eq 'WINE';

    my $type = ('zone', 'appel', 'ext')[$action];

    # Create new db item
    my $id = -1;
    my $name = '';
    if ($type eq 'zone') {
	$name = __u('New Zone');
	$id = $db->zone->add(Gwine::GZone->new($name, $parent_id));
    }
    elsif ($type eq 'appel') {
	$name = __u('New Appellation');
	$id = $db->appel->add(Gwine::GAppel->new($name, $parent_id));
    }
    elsif ($type eq 'ext') {
	$name = __u('New Extension');
	my $appel = $db->appel->by_id($parent_id);
	$id = $appel->add_extension($name);
	$id = "$parent_id:$id";
    }
    else { return }

    # Insert new cell
    my $iter = $store->append($iparent);
    $store->set($iter,
		0 => $pixmaps{$type},
		1 => $name,
		2 => "$type/$id");

    # if parent not expanded, expand !
    my $par_path = $store->get_path($iparent);
    $view->expand_row($par_path, FALSE);

    # Editing for new name
    my $path = $store->get_path($iter);
    my $column = $view->get_column(1);
    $view->set_cursor($path, $column, TRUE);
#    $view->grab_focus;
}

sub on_modify_zone_event {
    my $view = $gxml->get_widget('ztreeview');
    my $store = $view->get_model;
    my (undef, $iter) = $view->get_selection->get_selected;
    my $path = $store->get_path($iter);
    my $column = $view->get_column(1);

    $view->set_cursor($path, $column, TRUE);
#    $view->grab_focus;
}

sub on_delete_zone_event {
    my $store = $gxml->get_widget('ztreeview')->get_model;
    my $select = $gxml->get_widget('ztreeview')->get_selection;
    my (undef, $iter) = $select->get_selected;
    my ($type, $id) = split m!/!, $store->get_value($iter, 2);
    
    # if a subnode exists, cannot delete this node
    if ($store->iter_has_child($iter)) {
	my $msg = __u('This zone cannot be deleted: a sub-zone exists');
	$gxml->get_widget('appbar')->set_status($msg);
	if ($config->conf_popups) {
	    my $popup = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),
						 'destroy_with_parent',
						 'info', 'close',
						 $msg);
	    $popup->run;
	    $popup->destroy
	}
	return
    }

    # if a wine exists, cannot delete this node
    if ($type ne 'zone') {
	my $msg = __u('This appellation cannot be deleted: wines exist');
	my $iter = $gxml->get_widget('wtreeview')->get_model->get_iter_first;
	if (defined $iter) {
	    $gxml->get_widget('appbar')->set_status($msg);
	    if ($config->conf_popups) {
		my $popup = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),
						     'destroy_with_parent',
						     'info', 'close',
						     $msg);
		$popup->run;
		$popup->destroy
	    }
	    return;
	}
    }
    
    if ($config->conf_popups) {
	my $name;
	if ($type eq 'zone') { $name = $db->zone->by_id($id)->zone }
	elsif ($type eq 'appel') { $name = $db->appel->by_id($id)->appellation }
	elsif ($type eq 'ext') {
	    my $app = $db->appel->by_id($id);
	    my (undef, $ext) = split /:/, $id;
	    $name = $app->get_extension($ext)
	}
	my $msg = __xu("Zone '{zone}' will be deleted permanently. Are you sure?",
		      zone => $name);
	my $dialog = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),,
					      'destroy-with-parent',
					      'question', 'none', $msg);
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Delete zone from list...
    $store->remove($iter);
    # ... and from db
    my $name;
 SWITCH: {
	if ($type eq 'zone') { 
	    $name =$db->zone->by_id($id)->zone; 
	    $db->zone->del_by_id($id); 
	    last 
	}
	if ($type eq 'appel') { 
	    $name = $db->appel->by_id($id)->appellation;
	    $db->appel->del_by_id($id); 
	    last 
	}
	if ($type eq 'ext') { 
	    my ($appid, $extid) = split /:/, $id;
	    $name = $db->appel->by_id($id)->get_extension($extid);
	    $db->appel->by_id($appid)->delete_extension($extid);
	    last 
	}
    }
    $gxml->get_widget('appbar')->set_status(__xu('{file} deleted', 
						file => $name));
}


###################################################################
# Gwine/Wines menu
###################################################################


my $siter;
my $search_appel = sub {
    my ($model, undef, $iter, $data) = @_;

    my ($type, $appel) = split m!/!, $model->get($iter, 2);
    return FALSE if $type eq 'zone';
    if ($appel eq $data) {
	$siter = $iter->copy;
	return TRUE
    }
    else { undef $siter }
    return FALSE
};


sub on_new_wine_event {
    # Verify at least one appellation exists
    if ($db->appel->true_size == 0) {
	my $mes = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),
					   'modal', 'info', 'close',
					   __u('No appellation exists. You cannot create a wine without appellation. Select Tree view in View menu and select Wines node in tree. Thus, create zone then appellation with Gwine menu.'));
	$mes->run;
	$mes->destroy;
	return
    }

    my @prods = (__u('No producer'), sort $db->list_prod);
    my $dw = DialogWine->create($config->xml_path, $db, undef,
				title => __u("New wine"),
				xml => $gxml,
			        entry_size => 0.75,
				cb_color => [0, @wine_colors],
			        cb_prod => [0, @prods]);

    my $path = $config->icon_path;
    $dw->xml->get_widget('dialog_wine')->set_icon_from_file("$path/gwine.png");
    # Get Appellation from selected node in Zone Tree
    my $store = $gxml->get_widget('ztreeview')->get_model;
    my $select = $gxml->get_widget('ztreeview')->get_selection;
    my (undef, $iter) = $select->get_selected;
    my ($type,$id) =  split m!/!, $store->get_value($iter, 2);
    if (($type eq 'appel') || ($type eq 'ext')) {
	my $local_treeview = $dw->xml->get_widget('ztreeview2');
	my $local_store = $local_treeview->get_model;
	$local_store->foreach($search_appel, $id);
	if (defined $siter) {
	    $local_treeview->expand_to_path($local_store->get_path($siter));
	    $local_treeview->get_selection->select_iter($siter);
	}
	else { print "siter not defined!\n" }
    }
}

sub on_modify_wine_event {
    my $store = $gxml->get_widget('wtreeview')->get_model;
    my $select = $gxml->get_widget('wtreeview')->get_selection;
    my (undef, $iter) = $select->get_selected;
    my $wine_domain = $store->get_value($iter, 0);
    my $wine_id =  $store->get_value($iter, 9);
    my $wine = $db->wine->by_id($wine_id);

    my $title= __xu("Modify {object} (Id {id})",
		   object => $wine_domain,
		   id => $wine_id);
    my @prods = (__u('No producer'), sort $db->list_prod);
    my $prod = 0;
    foreach (@prods) {
	next if $wine->producer eq -1;
	last if ($_ eq $db->prod->by_id($wine->producer)->name);
	$prod++
    }
    my @entries = $wine->book_get_entries;
    my @cellar = ();
    foreach (@entries) {
	my $type = $_->{number} > 0 ? __u('In') : __u('Out');
	my $number = abs $_->{number};
	push @cellar, [$_->{date}, $type, $number, $_->{comment}];
    }
    my $dw = DialogWine->create($config->xml_path, $db, $wine_id,
				title => $title,
				xml => $gxml,
				entry_domain => $wine->domain,
				entry_vintage => $wine->millesime,
				cb_color => [$wine->color, @wine_colors],
				entry_quantity => $wine->number,
			        entry_size => $wine->size,
				txtview_notes => $wine->notes,
				entry_bdate => $wine->date,
				entry_cons => $wine->conservation,
				entry_price => $wine->price,
			        cb_prod => [$prod, @prods],
			        cellar_book => \@cellar);

    my $path = $config->icon_path;
    $dw->xml->get_widget('dialog_wine')->set_icon_from_file("$path/gwine.png");

    # Get wine-to-modify appellation to expand tree in DialogWine
    (undef, $iter) = $select->get_selected;
    if (defined $iter) {
	my $appel = $db->wine->by_id($wine_id)->appellation;
	my $local_treeview = $dw->xml->get_widget('ztreeview2');
	my $local_store = $local_treeview->get_model;
	$local_store->foreach($search_appel, $appel);
	if (defined $siter) {
	    $local_treeview->expand_to_path($local_store->get_path($siter));
	    $local_treeview->get_selection->select_iter($siter);
	}
	else { print "siter not defined!\n" }
    }
}

sub on_delete_wine_event {
    my $store = $gxml->get_widget('wtreeview')->get_model;
    my $select = $gxml->get_widget('wtreeview')->get_selection;
    my (undef, $iter) = $select->get_selected;
    my $wine_id =  $store->get_value($iter, 9);
    
    if ($config->conf_popups) {
	# Popus Dialog "Yes/No"
	my $msg = __xu("Wine '{wine}' will be deleted permanently. Are you sure?",
		      wine => $db->wine->by_id($wine_id)->domain);
	my $dialog = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),
					      'destroy-with-parent',
					      'question', 'none', $msg);
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Delete wine from list...
    $store->remove($iter);
    # ... and from db
    $db->wine->del_by_id($wine_id);    
}


###################################################################
# Gwine/Producers menu
###################################################################

sub on_new_prod_event {
    my $dp = DialogProd->create($config->xml_path, $db);
    $dp->xml->get_widget('dialog_prod')->set_title(__u("New Producer"));
    my $path = $config->icon_path;
    $dp->xml->get_widget('dialog_prod')->set_icon_from_file("$path/gwine.png");
}

sub on_modify_prod_event {
    my $store = $gxml->get_widget('ptreeview')->get_model;
    my $select = $gxml->get_widget('ptreeview')->get_selection;
    my $iter = $select->get_selected;
    my $prod_name = $store->get_value($iter, 0);
    my $prod_id =  $store->get_value($iter, 6);
    my $prod = $db->prod->by_id($prod_id);

    my $dp = DialogProd->create($config->xml_path, $db, $prod_id);
    my $path = $config->icon_path;
    $dp->xml->get_widget('dialog_prod')->set_icon_from_file("$path/gwine.png");
    my $title = __xu("Modify {object} (Id {id})", 
		    object => $prod_name,
		    id => $prod_id);
    my $dxml = $dp->xml;
    $dxml->get_widget('dialog_prod')->set_title($title);

    $dxml->get_widget('entry_name')->set_text($prod->name);
    $dxml->get_widget('entry_addr')->set_text($prod->address);
    $dxml->get_widget('entry_zip')->set_text($prod->zip);
    $dxml->get_widget('entry_city')->set_text($prod->city);
    $dxml->get_widget('entry_country')->set_text($prod->country);
    $dxml->get_widget('entry_phone')->set_text($prod->phone);
}

sub on_delete_prod_event {
    my $store = $gxml->get_widget('ptreeview')->get_model;
    my $select = $gxml->get_widget('ptreeview')->get_selection;
    my (undef, $iter) = $select->get_selected;
    my $prod_id =  $store->get_value($iter, 6);
    
    if ($config->conf_popups) {
	# Popus Dialog "Yes/No"
	my $msg = __xu("Producer '{prod}' will be deleted permanently. Are you sure?",
		       prod => $db->prod->by_id($prod_id)->name);
	my $dialog = Gtk2::MessageDialog->new($gxml->get_widget('app_gwine'),
					      'destroy-with-parent',
					      'question', 'none', $msg);
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Delete prod from list...
    $store->remove($iter);
    # ... and from db
    $db->prod->del_by_id($prod_id);    
}


######################################################################
## Zone Tree Signals and Functions
######################################################################

sub ztree_sort {
    my $self = shift;
    my ($itera, $iterb) = @_;

    my $a = $self->get_value($itera, 1);
    # On root level, no sorting is made : Wines node is always the 
    # first and Producers one always the last.
    return -1 if $a eq __u('Wines');
    return 1 if $a eq __u('Producers');
    my $b = $self->get_value($iterb, 1);
    return -1 if !defined $b;
    return 1 if !defined $a;
    return $a cmp $b
}

sub on_ztreeview_map {
    my $tview = shift;
    my $store = $tview->get_model;
    
    $store->clear;

    $wine_iter = $store->append(undef);
    $store->set($wine_iter,
		0 => $pixmaps{vines},
		1 => __u('Wines'),
		2 => 'WINE');
    $prod_iter = $store->append(undef);
    $store->set($prod_iter,
		0 => $pixmaps{prod},
		1 => __u('Producers'),
		2 => 'PROD');

    $store->sort_column_changed();

    my @lzone = $db->zone->list;
    my @zone_nodes = ();
    while (@lzone) {
	my @list = @lzone;
	@lzone = ();
	foreach my $zone (@list) {
	    if ($zone eq '-1') { next }
	    if ($zone->parent eq '-1') {
		my $id = $db->zone->get_id($zone);
		$zone_nodes[$id] = $store->append($wine_iter);
		$store->set($zone_nodes[$id],
			    0 => $pixmaps{zone},
			    1 => $zone->zone,
			    2 => "zone/$id");
	    }    
	    else {
		my $parent = $zone_nodes[$zone->parent];
		# Here, if parent is not still created, creation of child is delayed
		if (! defined $parent) { push @lzone, $zone; next }
		my $id = $db->zone->get_id($zone);
		$zone_nodes[$id] = $store->append($parent);
		$store->set($zone_nodes[$id], 
			    0 => $pixmaps{zone},
			    1 => $zone->zone,
			    2 => "zone/$id");
	    }
	}
    }
    my @lappel = $db->appel->list;
    foreach my $appel (@lappel) {
	if ($appel eq -1) { next }
	my $name = $appel->appellation;
	my $id = $db->appel->get_id($appel);
	my $appel_node = $store->append($zone_nodes[$appel->vineyard]);
	$store->set($appel_node, 
		    0 => $pixmaps{appel},
		    1 => $name,
		    2 => "appel/$id:");
	# For this appellation, we create its extensions
	if (defined $appel->extensions) {
	    foreach my $extension (sort @{$appel->extensions}) {
		if ($extension ne 'null') {
		    my $ext_id = $appel->get_extension_id( $extension );
		    $store->set($store->append($appel_node),
				0 => $pixmaps{ext},
				1 => $extension,
				2 => "ext/$id:$ext_id");
		}
	    }
	}
    }
    # Expand first Wines node
    $tview->expand_row($store->get_path($wine_iter), FALSE);
    # Re-select user's style view
#    $group->[2 - $config->view_style($old_style)]->activate; 
}

sub on_ztreeview_button_press_event {
    my ($class, $event, $data) = @_;

    my $store = $class->get_model;
    my $select = $class->get_selection;
    my (undef, $iter) = $select->get_selected;

    return 0 if !defined $iter;

    if ($event->button == 3) {
	my $type = $store->get_value($iter, 2);
	my $menu;
	if ($type eq 'PROD') {
	    $menu = $gxml->get_widget('producers_menu');
	}
	else {
	    $menu = $gxml->get_widget('zones_menu');
	}
	$menu->popup(undef, undef, undef, undef, 
		     $event->button, $event->time);
	return 1
    }
    else { # Selection
	$config->last_clicked('zonetree');
    }
    return 0;
}

sub on_ztreeview_selection_changed {
    my $widget = shift;
    
    my (undef, $iter) = $widget->get_selected;
    my $store = $widget->get_tree_view->get_model;

    return 0 if !defined $iter;
    my $path = $store->get_path($iter)->to_string;
    my $root_type = $store->get_value($iter, 2);
    
    my $wmenu = $gxml->get_widget('wines_menu');
    my $wmod = $gxml->get_widget('modify_wine');
    my $wdel = $gxml->get_widget('delete_wine');
    my $pmenu = $gxml->get_widget('producers_menu');
    my $pmod = $gxml->get_widget('modify_prod');
    my $pdel = $gxml->get_widget('delete_prod');
    my $zmenu = $gxml->get_widget('zones_menu');
    my $znew = $gxml->get_widget('new_zone');
    my $znew_zone = $gxml->get_widget('new_geographical_zone');
    my $znew_app = $gxml->get_widget('new_appellation');
    my $znew_ext = $gxml->get_widget('new_extension');
    my $zmod = $gxml->get_widget('modify_zone');
    my $zdel = $gxml->get_widget('delete_zone');

    # When selection changed in tree, in new wine or prod list, nothing
    # is selected. so we cannot delete or modify an item.
    $wmod->set_sensitive(FALSE);
    $wdel->set_sensitive(FALSE);
    $pmod->set_sensitive(FALSE);
    $pdel->set_sensitive(FALSE);

    # Producer's iter ?
    if ($root_type eq 'PROD') {
	$gxml->get_widget('list_label')->set_text(__u('Producers'));
	$gxml->get_widget('notebook1')->set_current_page(1);
	list_prods();
	# Set menus
	$wmenu->set_sensitive(FALSE);
	$pmenu->set_sensitive(TRUE);
	$zmenu->set_sensitive(FALSE);	
	return 0;
    }
    else {
	$gxml->get_widget('list_label')->set_text(__u('Wines'));
	list_wines();
 	$gxml->get_widget('notebook1')->set_current_page(0);
   }

    # Get Zone column to show or hide it
    my $nb = @wine_cols - 2;
    my $col = $gxml->get_widget('wtreeview')->get_column($nb);
    
    if ($root_type eq 'WINE') {
	$col->set_visible(TRUE); # Show Zone column
	# Set menus
	$wmenu->set_sensitive(TRUE);
	$pmenu->set_sensitive(FALSE);
	# if only wine list is showed, deactivate zone menu
	# else (tree view), zone menu is activated
	if ($config->view_style == 2) {
	    $zmenu->set_sensitive(TRUE);
	}
	else {
	    $zmenu->set_sensitive(FALSE);
	}
	$znew->set_sensitive(TRUE);
	$znew_zone->set_sensitive(TRUE);
	$znew_app->set_sensitive(FALSE);
	$znew_ext->set_sensitive(FALSE);
	$zmod->set_sensitive(FALSE);	
	$zdel->set_sensitive(FALSE);	
    }
    else {
	$col->set_visible(FALSE); # Hide Zone column
	my $name = $store->get_value($iter, 1);
	my ($type, $id) = split m!/!, $store->get_value($iter, 2);
	my $label = $gxml->get_widget('list_label');
	if ($type eq 'ext') {
	    my ($appel_id) = split /:/, $id;
	    my $appel = $db->appel->by_id($id);
	    my $appellation = $appel->appellation;
	    my $zone = $db->get_zone($appel->vineyard);
	    my $zones = $zone->zone;
	    while ($zone->parent != -1) {
		$zone = $db->get_zone($zone->parent);
		$zones = $zone->zone . '/' . $zones;
	    }
	    $label->set_text("$zones/$appellation $name");
	    list_wines();
	    $gxml->get_widget('notebook1')->set_current_page(0);
	    $col->set_visible(FALSE); # Hide Zone Column
	    # Set menus
	    $wmenu->set_sensitive(TRUE);
	    $pmenu->set_sensitive(FALSE);
	    $zmenu->set_sensitive(TRUE);	
	    $znew->set_sensitive(FALSE);
	    $zmod->set_sensitive(TRUE);	
	    $zdel->set_sensitive(TRUE);	
	}
	elsif ($type eq 'appel') {
	    my $appel = $db->appel->by_id($id);
	    my $zone = $db->get_zone($appel->vineyard);
	    my $zones = $zone->zone;
	    while ($zone->parent != -1) {
		$zone = $db->get_zone($zone->parent);
		$zones = $zone->zone . '/' . $zones;
	    }
	    $label->set_text("$zones/$name");
	    list_wines();
	    $gxml->get_widget('notebook1')->set_current_page(0);
	    $col->set_visible(FALSE); # Hide Zone Column
	    # Set menus
	    $wmenu->set_sensitive(TRUE);
	    $pmenu->set_sensitive(FALSE);
	    $zmenu->set_sensitive(TRUE);	
	    $znew->set_sensitive(TRUE);
	    $znew_zone->set_sensitive(FALSE);	
	    $znew_app->set_sensitive(FALSE);	
	    $znew_ext->set_sensitive(TRUE);	
	    $zmod->set_sensitive(TRUE);
	    $zdel->set_sensitive(TRUE);
	}
	elsif ($type eq 'zone') {
	    my $zone = $db->get_zone($id);
	    my $zones = $name;
	    while ($zone->parent != -1) {
		$zone = $db->get_zone($zone->parent);
		$zones = $zone->zone . '/' . $zones;
	    }
	    $label->set_text($zones);
	    $gxml->get_widget('notebook1')->set_current_page(2);
	    # Set menus
	    $wmenu->set_sensitive(FALSE);
	    $pmenu->set_sensitive(FALSE);
	    $zmenu->set_sensitive(TRUE);	
	    $znew->set_sensitive(TRUE);
	    $znew_zone->set_sensitive(TRUE);	
	    $znew_app->set_sensitive(TRUE);	
	    $znew_ext->set_sensitive(FALSE);	
	    $zmod->set_sensitive(TRUE);
	    $zdel->set_sensitive(TRUE);
	}
    }
    return 0;
}

sub on_renderer_text_edited_event {
    my $cell = shift;
    my $path = shift;
    my $new_name = shift;
    
    my $store = $gxml->get_widget('ztreeview')->get_model;
    my $iter = $store->get_iter_from_string($path);
    return 0 if !defined $iter;

    my ($type,$id) = split m!/!, $store->get_value($iter, 2);

    if ($type eq 'zone') {
	# Zone
	my $zone = $db->zone->by_id($id);
	$zone->zone($new_name);
    }
    elsif ($type eq 'appel') {
	# Appellation
	my ($appel_id) = split /:/, $id;
	my $appel = $db->appel->by_id($appel_id);
	$appel->appellation($new_name);
    }
    elsif ($type eq 'ext') {
	# Extension
	my ($appel_id, $ext_id) = split /:/, $id;
	my $appel = $db->appel->by_id($appel_id);
	$appel->set_extension_id($ext_id, $new_name);
    }
    else { return }

    $store->set($iter, 1, $new_name);
}


######################################################################
## Wine List Signals and Functions
######################################################################

# Add a Gwine::GWine in Wine List
sub add_wine {
    my ($wine, $db) = @_;

    my $pid = $wine->producer;
    my $prod = '';
    if ($pid != -1) {
	my $p = $db->prod->by_id($pid);
	$prod = $p->name if $p ne '-1';
    }
    my $wine_id = $db->wine->get_id($wine);

    my $store = $gxml->get_widget('wtreeview')->get_model;
    my $ins_iter = $store->append;
    $store->set($ins_iter,
		0 => $wine->domain,
		1 => $wine->millesime,
		2 => $wine_colors[$wine->color],
		3 => $wine->real_number,
		4 => $lh->numf($wine->price),
		5 => $wine->date,
		6 => $lh->numf($wine->size, 0),
		7 => $prod,
		8 => $wine->data,
		9 => $wine_id);
}

sub list_wines {
    my $list_store = $gxml->get_widget('wtreeview')->get_model;
    my $tree_store = $gxml->get_widget('ztreeview')->get_model;

    my (undef, $iter) = $gxml->get_widget('ztreeview')->get_selection->get_selected;

    # Go back if $iter is not defined
    return 0 if !defined $iter;
    # Flush list
    $list_store->clear;

    my ($type, $id) = split m!/!, $tree_store->get_value($iter, 2);
    
    # Go back if Producer node is selected
    return 0 if $type eq 'PROD';

    # Appellation node selected (include extension nodes)?
    my ($appel_ref, $appel_num, $appel_str);

    my @lwine = $db->wine->list;
    foreach my $wine (@lwine) {
	next if ($wine eq -1);
	# If hide_wines is true, wines with no bottle are hidden
	next if ($config->hide_wines && $wine->real_number == 0);
	# Add wine if selected node is WINE node or if wine's appellation 
	# corresponds to selected node
	if (($type eq 'WINE') or ($wine->appellation eq $id)) {
	    add_wine($wine, $db)
	}
    }

    # Sort wine list
    $list_store->set_sort_column_id($config->wlist_sort_column,
				    $config->wlist_sort_order);

    return 0
}


sub wlist_sort {
    my $self = shift;
    my ($itera, $iterb) = @_;

    my $a = $self->get_value($itera, $config->wlist_sort_column);
    my $b = $self->get_value($iterb, $config->wlist_sort_column);

    if (!defined $a) { return 1 }
    elsif (!defined $b) { return -1 }

    if ($config->wlist_sort_column == 5) {
	# Sorting by date
	return $lh->sort_by_date($a, $b)
    }
    else { # Sorting
	my $ctype = $self->get_column_type($config->wlist_sort_column);
	return $a <=> $b if $ctype eq 'Glib::Int';
	return $a cmp $b
    }
}

sub on_wtreeview_column_clicked {
    my $self = shift;
    my $col_nb = shift;
    if ($config->wlist_sort_column == $col_nb) {
	if ($config->wlist_sort_order eq 'ascending') {
	    $config->wlist_sort_order('descending')
	} 
	else { $config->wlist_sort_order('ascending') }
    } 
    else { $config->wlist_sort_column($col_nb) }
    my $store = $gxml->get_widget('wtreeview')->get_model;
    $store->set_sort_column_id($col_nb, $config->wlist_sort_order);
}

sub on_wtreeview_button_press_event {
    my ($class, $event, $data) = @_;

    my $store = $class->get_model;
    my $select = $class->get_selection;
    my (undef, $iter) = $select->get_selected;

    if ($event->button == 3) { # Left mouse click (or middle...)
	my $menu = $gxml->get_widget('wines_menu');
#	$menu = $menu->get_submenu;
	$menu->popup(undef, undef, undef, undef, 
		     $event->button, $event->time);
	return 1
    }
    else { # Selection
	$config->last_clicked('winelist');
    }
    return 0;
}


sub on_wtreeview_selection_changed {
    my ($class, $event, $data) = @_;
    
    my (undef, $iter) = $class->get_selected;
    my $store = $class->get_tree_view->get_model;
    if (!defined $iter) {
	# If $iter is undefined, there's nothing selected
	# so, we cannot modify or delete anything
	$gxml->get_widget('modify_wine')->set_sensitive(FALSE);
	$gxml->get_widget('delete_wine')->set_sensitive(FALSE);
	return 0
    }
    else {
	$gxml->get_widget('modify_wine')->set_sensitive(TRUE);
	$gxml->get_widget('delete_wine')->set_sensitive(TRUE);
    }
    my $path = $store->get_path($iter)->to_string;
    
}


######################################################################
## Producer List Signals and Functions
######################################################################

# Add a Gwine::GProd in Producer List
sub add_prod {
    my ($prod, $db) = @_;

    my $store = $gxml->get_widget('ptreeview')->get_model;
    my $ins_iter = $store->append;
    my $id = $db->prod->get_id($prod);
    $store->set($ins_iter,
		0 => $prod->name,
		1 => $prod->address,
		2 => $prod->zip,
		3 => $prod->city,
		4 => $prod->country,
		5 => $prod->phone,
		6 => $id);
}

sub list_prods {
    my $list_store = $gxml->get_widget('ptreeview')->get_model;

    # Flush list
    $list_store->clear;

    # Set prod items in list
    my @lprod = $db->prod->list;
    foreach my $prod (@lprod) {
	next if ($prod eq -1);
	add_prod($prod, $db);
    }
    return 0;
}


sub plist_sort {
    my $self = shift;
    my ($itera, $iterb) = @_;

    my $a = $self->get_value($itera, $config->plist_sort_column);
    my $b = $self->get_value($iterb, $config->plist_sort_column);
    return $a cmp $b
}

sub on_ptreeview_column_clicked {
    my $self = shift;
    my $col_nb = shift;
    if ($config->plist_sort_column == $col_nb) {
	if ($config->plist_sort_order eq 'ascending') {
	    $config->plist_sort_order('descending')
	} 
	else { $config->plist_sort_order('ascending') }
    } 
    else { $config->plist_sort_column($col_nb) }
    
    my $store = $gxml->get_widget('ptreeview')->get_model;
    $store->set_sort_column_id($col_nb, $config->plist_sort_order);
}

sub on_ptreeview_button_press_event {
    my ($class, $event, $data) = @_;

    my $store = $class->get_model;
    my $select = $class->get_selection;
    my (undef, $iter) = $select->get_selected;

    my $menu = $gxml->get_widget('producers_menu');
    if ($event->button == 3) { # Left mouse click (or middle...)
	$menu->popup(undef, undef, undef, undef, 
		     $event->button, $event->time);
	return 1
    }
    else { # Selection
	$config->last_clicked('prodlist');
    }
    return 0;
}


sub on_ptreeview_selection_changed {
    my ($class, $event, $data) = @_;
    
    my (undef, $iter) = $class->get_selected;
    my $store = $class->get_tree_view->get_model;

    if (!defined $iter) {
	# If $iter is undefined, there's nothing selected
	# so, we cannot modify or delete anything
	$gxml->get_widget('modify_prod')->set_sensitive(FALSE);
	$gxml->get_widget('delete_prod')->set_sensitive(FALSE);
	return 0
    }
    else {
	$gxml->get_widget('modify_prod')->set_sensitive(TRUE);
	$gxml->get_widget('delete_prod')->set_sensitive(TRUE);
    }
}




1;

__END__
